/*=========================================================================

  Program:   Visualization Toolkit
  Module:    vtk_cli11_external.h.in

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
#ifndef vtk_cli11_external_h
#define vtk_cli11_external_h

/**
 * Don't include this header directly. Use `vtk_cli11_forward.h` or
 * `vtk_cli11.h` instead.
 */

/* Use the CLI11 library configured for VTK.  */
#define VTK_MODULE_USE_EXTERNAL_VTK_cli11 0

#endif
